DEFAULT_EXTRACT_ELEMENTS_TIMEOUT_SECONDS = 300
DEFAULT_EXTRACT_DATA_TIMEOUT_SECONDS = 900
DEFAULT_WAIT_FOR_NETWORK_IDLE = True
DEFAULT_INCLUDE_HIDDEN_DATA = True
DEFAULT_INCLUDE_HIDDEN_ELEMENTS = False
DEFAULT_RESPONSE_MODE = "fast"

DEFAULT_WAIT_FOR_PAGE_LOAD_SECONDS = 0
DEFAULT_IS_SCROLL_TO_BOTTOM_ENABLED = False
DEFAULT_IS_SCREENSHOT_ENABLED = False
DEFAULT_IS_STEALTH_MODE_ENABLED = False

EXTRACT_DATA_ENDPOINT = "https://api.agentql.com/v1/query-data"
DEFAULT_API_TIMEOUT_SECONDS = 900

REQUEST_ORIGIN = "llamaindex"
