QUERY_PROMPT_REQUIRED_ERROR_MESSAGE = (
    "Invalid arguments provided. Either 'query' or 'prompt' must be provided."
)
QUERY_PROMPT_EXCLUSIVE_ERROR_MESSAGE = (
    "Invalid arguments provided. Only one of 'query' or 'prompt' should be provided."
)
UNSET_API_KEY_ERROR_MESSAGE = "No AgentQL API key provided. You can set your API key in code by specifying the `api_key` argument or by setting the `AGENTQL_API_KEY` environment variable. You can create an API key at https://dev.agentql.com."
UNAUTHORIZED_ERROR_MESSAGE = "Invalid AgentQL API key provided. Please provide a valid API Key. You can create one at https://dev.agentql.com."
